#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

from cve_manager.common import GetDef, Init as CommonInit
from cve_manager.conf   import DB_CON_SEC
from cpe_map.mediator   import Mediator
from cpe_map.speaker    import Speaker

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Creating the intermediary objects

def Init(type_of_matching, args):

	NECESSARY_ATTRIBUTES = ('noupdate',)

	if any(not hasattr(args, attr) for attr in NECESSARY_ATTRIBUTES):
		raise f'Can\'t initialize a module for "{type_of_matching}" ' \
			'matching, some arguments are missing'
		return [None] * 3

	is_bin = args.bin if hasattr(args, 'bin') else False

	printer, conf = CommonInit(args)
	mysql_config, = conf.Get([DB_CON_SEC])
	conf_dir = GetDef('CONF_DIR', args.debug)

	speaker = Speaker(
		type_of_matching,
		printer,
		is_bin,
		args.noupdate
		)

	mediator = Mediator(
		type_of_matching,
		mysql_config,
		conf_dir,
		printer,
		is_bin,
		args.noupdate
		)

	return (conf, speaker, mediator)
