#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

from mysql.connector import Error as MySQLError
from cve_manager.db  import DB
from cpe_map         import defines as df

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

class Worker():

	def __init__(self, mysql_config, printer, types_of_products, branches):

		self.mysql_config = mysql_config
		self.printer = printer
		self.types_of_products = types_of_products
		self.branches = branches

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	# Create a mapping table for the branch

	def __CreateTablesForMatches(self, type_of_products, src_or_bin, recreate):

		if src_or_bin not in ('src', 'bin'):
			this = self.__CreateTablesForMatches
			return (
				this(type_of_products, 'src', recreate) and
				this(type_of_products, 'bin', recreate)
				)

		# Connecting to the CVE database
		db = DB(self.printer)
		if not db.Connect(self.mysql_config):
			return False

		target_table = f'{src_or_bin}_to_{type_of_products}_map'

		self.printer.LineBegin(f'Creating "{target_table}" table')

		# Not recreating the table if already exist and there is no spec request
		if db.TableExists(target_table) and not recreate:
			self.printer.LineEnd('[SKIPPING: Already exist]')
			return True

		# Not recreating the table if there is no corresponding tables
		for branch in self.branches:
			src_or_bin_table = branch + '_' + src_or_bin
			vul_table = type_of_products + '_vul_import'
			if db.TableExists(src_or_bin_table) and db.TableExists(vul_table):
				continue
			msg = f'To create "{target_table}" table "{src_or_bin_table}" ' \
				f'and "{vul_table}" tables should already exist'
			self.printer.Err(msg)
			return False

		# Forming the table description
		VARCHAR = f'VARCHAR({df.NAME_MAX_LEN})'
		columns = [f'package {VARCHAR} COLLATE utf8mb4_0900_as_cs']
		if src_or_bin == 'src':
			columns += [
				f'{df.M_PRESCRIBED} {VARCHAR}',
				f'{df.M_COMPLETE} {VARCHAR}',
				f'{df.M_PARTIAL} LONGTEXT',
				]
			for branch in self.branches:
				columns += [
					f'{branch}_{df.M_URL_GRADE_A} TEXT',
					f'{branch}_{df.M_URL_GRADE_B} TEXT',
					f'{branch}_{df.M_FIXES} TEXT',
					]
		else:
			columns += [
				f'{df.M_BIN_COMPLETE} {VARCHAR}',
				f'{df.M_BIN_PARTIAL} LONGTEXT',
				]
		columns.append(f'CONSTRAINT pk_{target_table} PRIMARY KEY (package)')

		if not db.RecreateTable(target_table, columns):
			return False

		self.printer.Success()

		return True

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	# Create tables for intermediate results of modules of various types
	# of matching

	def Prepare(self, recreate):

		for t in self.types_of_products:
			if not self.__CreateTablesForMatches(t, 'src_and_bin', recreate):
				return False

		return True
