#!/usr/bin/python3

#	cve-manager : CVE management tool
#	Copyright (C) 2017-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

from cve_manager.conf import LOCAL_SYS

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Speaker obj performs all printing on a top level of all cpe-map-* modules

class Speaker():

	def __init__(self, type_of_matching, printer, target_is_bin, verbose):

		self.type_of_matching = type_of_matching
		self.printer = printer
		self.target_is_bin = target_is_bin
		self.verbose = verbose

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	# Announcing a matching operation or a given custom operation

	def Op(self, data_source, packages, products, branch=None):

		if any(all(isinstance(names, t) for names in packages) for t in
				(list, dict)):
			n_packages = sum([len(section) for section in packages])
		else:
			n_packages = len(packages)
		n_products = len(products)

		msg = f'Searching for "{self.type_of_matching}" matches of ' \
			f'{n_packages} {"bin" if self.target_is_bin else "src"} packages '
		if branch:
			msg += 'of ' + ('the local system' if branch == LOCAL_SYS else
				f'{branch} branch') + ' '
		msg += f'with {n_products} {data_source.upper()} products'
		self.printer.LineBegin(msg)

	def Say(self, msg, endl=False):
		if endl:
			self.printer.LineEnd(msg)
		else:
			self.printer.LineBegin(msg)

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	# Print a status message

	def Status(self, success=None, warn=None, err=None, extra=[], extra_warn=[],
			matches=None):

		for line in extra_warn:
			self.printer.LineAddWarn(line)

		for line in extra:
			self.printer.LineAddExtra(line)

		if err != None:
			self.printer.Err(err)
		else:
			if matches:
				n_matches = len(matches)
				success = f'Got {n_matches} matches' if n_matches > 0 \
					else 'No matches'
				if self.verbose:
					self.Matches(matches, True)
			if warn == None or success:
				self.printer.Success(success)
			if warn:
				self.printer.Warn(warn)

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	# Print given matches

	def Matches(self, matches, as_extra=False):

		report = self.printer.LineAddExtra if as_extra else self.printer.LineEnd

		for name, match in matches.items():
			report(f'{name} >> {match}')
